@@ -136,6 +136,8 @@ static void                  g_key_file_add_key                (GKeyFile
 								const gchar            *value);
 static void                  g_key_file_add_group              (GKeyFile               *key_file,
 								const gchar            *group_name);
+static gboolean              g_key_file_is_group_name          (const gchar *name);
+static gboolean              g_key_file_is_key_name            (const gchar *name);
 static void                  g_key_file_key_value_pair_free    (GKeyFileKeyValuePair   *pair);
 static gboolean              g_key_file_line_is_comment        (const gchar            *line);
 static gboolean              g_key_file_line_is_group          (const gchar            *line);
@@ -260,6 +262,8 @@ void
 g_key_file_set_list_separator (GKeyFile *key_file,
 			       gchar     separator)
 {
+  g_return_if_fail (key_file != NULL);
+
   key_file->list_separator = separator;
 }
 
@@ -690,10 +694,14 @@ g_key_file_parse_line (GKeyFile     *key_file,
 				     &parse_error);
   else
     {
+      gchar *line_utf8 = _g_utf8_make_valid (line);
       g_set_error (error, G_KEY_FILE_ERROR,
                    G_KEY_FILE_ERROR_PARSE,
                    _("Key file contains line '%s' which is not "
-                     "a key-value pair, group, or comment"), line);
+                     "a key-value pair, group, or comment"), 
+		   line_utf8);
+      g_free (line_utf8);
+
       return;
     }
 
@@ -743,6 +751,9 @@ g_key_file_parse_group (GKeyFile     *key_file,
   group_name = g_strndup (group_name_start, 
                           group_name_end - group_name_start);
   
+  if (!g_key_file_is_group_name (group_name))
+    g_critical (_("Invalid group name: %s"), group_name);
+
   g_key_file_add_group (key_file, group_name);
   g_free (group_name);
 }
@@ -782,6 +793,9 @@ g_key_file_parse_key_value_pair (GKeyFile     *key_file,
 
   key = g_strndup (line, key_len - 1);
 
+  if (!g_key_file_is_key_name (key))
+    g_critical (_("Invalid key name: %s"), key);
+
   /* Pull the value from the line (chugging leading whitespace)
    */
   while (g_ascii_isspace (*value_start))
@@ -801,9 +815,12 @@ g_key_file_parse_key_value_pair (GKeyFile     *key_file,
     {
       if (g_ascii_strcasecmp (value, "UTF-8") != 0)
         {
+	  gchar *value_utf8 = _g_utf8_make_valid (value);
           g_set_error (error, G_KEY_FILE_ERROR,
                        G_KEY_FILE_ERROR_UNKNOWN_ENCODING,
-                       _("Key file contains unsupported encoding '%s'"), value);
+                       _("Key file contains unsupported "
+			 "encoding '%s'"), value_utf8);
+	  g_free (value_utf8);
 
           g_free (key);
           g_free (value);
@@ -945,6 +962,9 @@ g_key_file_to_data (GKeyFile  *key_file,
 
       if (group->comment != NULL)
         g_string_append_printf (data_string, "%s\n", group->comment->value);
+      else if (group_node->next) /* separate groups by at least an empty line */
+        g_string_append_c (data_string, '\n');
+
       if (group->name != NULL)
         g_string_append_printf (data_string, "[%s]\n", group->name);
 
@@ -1206,9 +1226,13 @@ g_key_file_set_value (GKeyFile    *key_file,
   GKeyFileKeyValuePair *pair;
 
   g_return_if_fail (key_file != NULL);
+  g_return_if_fail (value != NULL);
   g_return_if_fail (group_name != NULL);
   g_return_if_fail (key != NULL);
-  g_return_if_fail (value != NULL);
+  if (!g_key_file_is_group_name (group_name))
+    g_critical (_("Invalid group name: %s"), group_name);
+  if (!g_key_file_is_key_name (key))
+    g_critical (_("Invalid key name: %s"), key);
 
   group = g_key_file_lookup_group (key_file, group_name);
 
@@ -1277,11 +1301,14 @@ g_key_file_get_string (GKeyFile     *key_file,
 
   if (!g_utf8_validate (value, -1, NULL))
     {
+      gchar *value_utf8 = _g_utf8_make_valid (value);
       g_set_error (error, G_KEY_FILE_ERROR,
                    G_KEY_FILE_ERROR_UNKNOWN_ENCODING,
                    _("Key file contains key '%s' with value '%s' "
-                     "which is not UTF-8"), key, value);
+                     "which is not UTF-8"), key, value_utf8);
+      g_free (value_utf8);
       g_free (value);
+
       return NULL;
     }
 
@@ -1331,8 +1358,6 @@ g_key_file_set_string (GKeyFile    *key_file,
   gchar *value;
 
   g_return_if_fail (key_file != NULL);
-  g_return_if_fail (group_name != NULL);
-  g_return_if_fail (key != NULL);
   g_return_if_fail (string != NULL);
 
   value = g_key_file_parse_string_as_value (key_file, string, FALSE);
@@ -1386,11 +1411,14 @@ g_key_file_get_string_list (GKeyFile     *key_file,
 
   if (!g_utf8_validate (value, -1, NULL))
     {
+      gchar *value_utf8 = _g_utf8_make_valid (value);
       g_set_error (error, G_KEY_FILE_ERROR,
                    G_KEY_FILE_ERROR_UNKNOWN_ENCODING,
                    _("Key file contains key '%s' with value '%s' "
-                     "which is not UTF-8"), key, value);
+                     "which is not UTF-8"), key, value_utf8);
+      g_free (value_utf8);
       g_free (value);
+
       return NULL;
     }
 
@@ -1454,8 +1482,6 @@ g_key_file_set_string_list (GKeyFile            *key_file,
   gsize i;
 
   g_return_if_fail (key_file != NULL);
-  g_return_if_fail (group_name != NULL);
-  g_return_if_fail (key != NULL);
   g_return_if_fail (list != NULL);
 
   value_list = g_string_sized_new (length * 128);
@@ -1498,7 +1524,6 @@ g_key_file_set_locale_string (GKeyFile     *key_file,
   gchar *full_key, *value;
 
   g_return_if_fail (key_file != NULL);
-  g_return_if_fail (group_name != NULL);
   g_return_if_fail (key != NULL);
   g_return_if_fail (locale != NULL);
   g_return_if_fail (string != NULL);
@@ -1584,7 +1609,7 @@ g_key_file_get_locale_string (GKeyFile     *key_file,
 						candidate_key, NULL);
       g_free (candidate_key);
 
-      if (translated_value && g_utf8_validate (translated_value, -1, NULL))
+      if (translated_value)
 	break;
 
       g_free (translated_value);
@@ -1701,7 +1726,6 @@ g_key_file_set_locale_string_list (GKeyFile            *key_file,
   gsize i;
 
   g_return_if_fail (key_file != NULL);
-  g_return_if_fail (group_name != NULL);
   g_return_if_fail (key != NULL);
   g_return_if_fail (locale != NULL);
   g_return_if_fail (length != 0);
@@ -1811,8 +1835,6 @@ g_key_file_set_boolean (GKeyFile    *key_file,
   gchar *result;
 
   g_return_if_fail (key_file != NULL);
-  g_return_if_fail (group_name != NULL);
-  g_return_if_fail (key != NULL);
 
   result = g_key_file_parse_boolean_as_value (key_file, value);
   g_key_file_set_value (key_file, group_name, key, result);
@@ -1917,8 +1939,6 @@ g_key_file_set_boolean_list (GKeyFile    *key_file,
   gsize i;
 
   g_return_if_fail (key_file != NULL);
-  g_return_if_fail (group_name != NULL);
-  g_return_if_fail (key != NULL);
   g_return_if_fail (list != NULL);
 
   value_list = g_string_sized_new (length * 8);
@@ -2027,8 +2047,6 @@ g_key_file_set_integer (GKeyFile    *key_file,
   gchar *result;
 
   g_return_if_fail (key_file != NULL);
-  g_return_if_fail (group_name != NULL);
-  g_return_if_fail (key != NULL);
 
   result = g_key_file_parse_integer_as_value (key_file, value);
   g_key_file_set_value (key_file, group_name, key, result);
@@ -2130,8 +2148,6 @@ g_key_file_set_integer_list (GKeyFile     *key_file,
   gsize i;
 
   g_return_if_fail (key_file != NULL);
-  g_return_if_fail (group_name != NULL);
-  g_return_if_fail (key != NULL);
   g_return_if_fail (list != NULL);
 
   values = g_string_sized_new (length * 16);
@@ -2241,10 +2257,8 @@ g_key_file_set_double  (GKeyFile    *key_file,
   gchar result[G_ASCII_DTOSTR_BUF_SIZE];
 
   g_return_if_fail (key_file != NULL);
-  g_return_if_fail (group_name != NULL);
-  g_return_if_fail (key != NULL);
 
-  g_ascii_dtostr ( result, sizeof (result), value );
+  g_ascii_dtostr (result, sizeof (result), value);
   g_key_file_set_value (key_file, group_name, key, result);
 }
 
@@ -2331,7 +2345,7 @@ g_key_file_get_double_list  (GKeyFile     *key_file,
  * @group_name.  If @key cannot be found then it is created.
  * If @group_name is %NULL the start group is used.
  *
- * Since: 2.21
+ * Since: 2.12
  **/
 void
 g_key_file_set_double_list (GKeyFile     *key_file,
@@ -2344,8 +2358,6 @@ g_key_file_set_double_list (GKeyFile     *key_file,
   gsize i;
 
   g_return_if_fail (key_file != NULL);
-  g_return_if_fail (group_name != NULL);
-  g_return_if_fail (key != NULL);
   g_return_if_fail (list != NULL);
 
   values = g_string_sized_new (length * 16);
@@ -2439,6 +2451,10 @@ g_key_file_set_group_comment (GKeyFile             *key_file,
 {
   GKeyFileGroup *group;
   
+  g_return_if_fail (group_name != NULL);
+  if (!g_key_file_is_group_name (group_name))
+    g_critical (_("Invalid group name: %s"), group_name);
+
   group = g_key_file_lookup_group (key_file, group_name);
   if (!group)
     {
@@ -2558,6 +2574,10 @@ g_key_file_get_key_comment (GKeyFile             *key_file,
   GString *string;
   gchar *comment;
 
+  g_return_val_if_fail (group_name != NULL, NULL);
+  if (!g_key_file_is_group_name (group_name))
+    g_critical (_("Invalid group name: %s"), group_name);
+
   group = g_key_file_lookup_group (key_file, group_name);
   if (!group)
     {
@@ -2870,9 +2890,15 @@ g_key_file_add_group (GKeyFile    *key_file,
 
   g_return_if_fail (key_file != NULL);
   g_return_if_fail (group_name != NULL);
+  if (!g_key_file_is_group_name (group_name))
+    g_critical (_("Invalid group name: %s"), group_name);
 
-  if (g_key_file_lookup_group_node (key_file, group_name) != NULL)
-    return;
+  group = g_key_file_lookup_group (key_file, group_name);
+  if (group != NULL)
+    {
+      key_file->current_group = group;
+      return;
+    }
 
   group = g_new0 (GKeyFileGroup, 1);
   group->name = g_strdup (group_name);
@@ -3179,6 +3205,58 @@ g_key_file_line_is_comment (const gchar *line)
   return (*line == '#' || *line == '\0' || *line == '\n');
 }
 
+static gboolean 
+g_key_file_is_group_name (const gchar *name)
+{
+  gchar *p, *q;
+
+  if (name == NULL)
+    return FALSE;
+
+  p = q = (gchar *) name;
+  while (*q && *q != ']' && *q != '[' && !g_ascii_iscntrl (*q))
+    q = g_utf8_next_char (q);
+  
+  if (*q != '\0' || q == p)
+    return FALSE;
+
+  return TRUE;
+}
+
+static gboolean
+g_key_file_is_key_name (const gchar *name)
+{
+  gchar *p, *q;
+
+  if (name == NULL)
+    return FALSE;
+
+  p = q = (gchar *) name;
+  /* We accept a little more than the desktop entry spec says,
+   * since gnome-vfs uses mime-types as keys in its cache.
+   */
+  while (*q && (g_unichar_isalnum (g_utf8_get_char (q)) || 
+                *q == '-' || *q == '_' || *q == '/' || *q == '+' || *q == '.'))
+    q = g_utf8_next_char (q);
+  
+  if (*q == '[')
+    {
+      q++;
+      while (*q && (g_unichar_isalnum (g_utf8_get_char (q)) || *q == '-' || *q == '_' || *q == '.' || *q == '@'))
+        q = g_utf8_next_char (q);
+
+      if (*q != ']')
+        return FALSE;     
+
+      q++;
+    }
+
+  if (*q != '\0' || q == p)
+    return FALSE;
+
+  return TRUE;
+}
+
 /* A group in a key file is made up of a starting '[' followed by one
  * or more letters making up the group name followed by ']'.
  */
@@ -3191,12 +3269,7 @@ g_key_file_line_is_group (const gchar *line)
   if (*p != '[')
     return FALSE;
 
-  p = g_utf8_next_char (p);
-
-  /* Group name must be non-empty
-   */
-  if (!*p || *p == ']')
-    return FALSE;
+  p++;
 
   while (*p && *p != ']')
     p = g_utf8_next_char (p);
@@ -3429,19 +3502,27 @@ g_key_file_parse_value_as_integer (GKeyFile     *key_file,
 
   if (*value == '\0' || *end_of_valid_int != '\0')
     {
+      gchar *value_utf8 = _g_utf8_make_valid (value);
       g_set_error (error, G_KEY_FILE_ERROR,
 		   G_KEY_FILE_ERROR_INVALID_VALUE,
-		   _("Value '%s' cannot be interpreted as a number."), value);
+		   _("Value '%s' cannot be interpreted "
+		     "as a number."), value_utf8);
+      g_free (value_utf8);
+
       return 0;
     }
 
   int_value = long_value;
   if (int_value != long_value || errno == ERANGE)
     {
+      gchar *value_utf8 = _g_utf8_make_valid (value);
       g_set_error (error,
 		   G_KEY_FILE_ERROR, 
 		   G_KEY_FILE_ERROR_INVALID_VALUE,
-		   _("Integer value '%s' out of range"), value);
+		   _("Integer value '%s' out of range"), 
+		   value_utf8);
+      g_free (value_utf8);
+
       return 0;
     }
   
@@ -3467,9 +3548,15 @@ g_key_file_parse_value_as_double  (GKeyFile     *key_file,
   double_value = g_ascii_strtod (value, &end_of_valid_d);
 
   if (*end_of_valid_d != '\0' || end_of_valid_d == value)
-    g_set_error (error, G_KEY_FILE_ERROR,
-                 G_KEY_FILE_ERROR_INVALID_VALUE,
-                 _("Value '%s' cannot be interpreted as a float number."), value);
+    {
+      gchar *value_utf8 = _g_utf8_make_valid (value);
+      g_set_error (error, G_KEY_FILE_ERROR,
+		   G_KEY_FILE_ERROR_INVALID_VALUE,
+		   _("Value '%s' cannot be interpreted "
+		     "as a float number."), 
+		   value_utf8);
+      g_free (value_utf8);
+    }
 
   return double_value;
 }
@@ -3479,6 +3566,8 @@ g_key_file_parse_value_as_boolean (GKeyFile     *key_file,
 				   const gchar  *value,
 				   GError      **error)
 {
+  gchar *value_utf8;
+
   if (value)
     {
       if (strcmp (value, "true") == 0 || strcmp (value, "1") == 0)
@@ -3487,9 +3576,12 @@ g_key_file_parse_value_as_boolean (GKeyFile     *key_file,
         return FALSE;
     }
 
+  value_utf8 = _g_utf8_make_valid (value);
   g_set_error (error, G_KEY_FILE_ERROR,
                G_KEY_FILE_ERROR_INVALID_VALUE,
-               _("Value '%s' cannot be interpreted as a boolean."), value);
+               _("Value '%s' cannot be interpreted "
+		 "as a boolean."), value_utf8);
+  g_free (value_utf8);
 
   return FALSE;
 }
