@@ -21,6 +21,29 @@ load_data (const gchar   *data,
   return keyfile;
 }
 
+static GLogLevelFlags log_flags = 0;
+
+static void 
+log_func (const gchar   *log_domain,
+          GLogLevelFlags log_level,
+          const gchar   *message,
+          gpointer       user_data)
+{
+  log_flags |= log_level;   
+}
+
+static void
+check_log (GLogLevelFlags flags)
+{
+  if ((log_flags & flags) == 0)
+    {
+      g_print ("Missing a log at level %d\n", flags);
+      exit (1);
+    }
+
+  log_flags = 0;
+}
+
 static void
 check_error (GError **error,
 	     GQuark   domain,
@@ -967,14 +990,13 @@ test_group_names (void)
   gchar *value;
 
   /* [ in group name */
+  log_flags = 0;
   data = "[a[b]\n"
          "key1=123\n";
   keyfile = g_key_file_new ();
   g_key_file_load_from_data (keyfile, data, -1, 0, &error);
   g_key_file_free (keyfile);  
-  check_error (&error, 
-	       G_KEY_FILE_ERROR,
-	       G_KEY_FILE_ERROR_PARSE);
+  check_log (G_LOG_LEVEL_CRITICAL);
 
   /* ] in group name */
   data = "[a]b]\n"
@@ -982,9 +1004,7 @@ test_group_names (void)
   keyfile = g_key_file_new ();
   g_key_file_load_from_data (keyfile, data, -1, 0, &error);
   g_key_file_free (keyfile);  
-  check_error (&error, 
-	       G_KEY_FILE_ERROR,
-	       G_KEY_FILE_ERROR_PARSE);
+  check_log (G_LOG_LEVEL_CRITICAL);
 
   /* control char in group name */
   data = "[a\tb]\n"
@@ -992,9 +1012,7 @@ test_group_names (void)
   keyfile = g_key_file_new ();
   g_key_file_load_from_data (keyfile, data, -1, 0, &error);
   g_key_file_free (keyfile);  
-  check_error (&error, 
-	       G_KEY_FILE_ERROR,
-	       G_KEY_FILE_ERROR_PARSE);
+  check_log (G_LOG_LEVEL_CRITICAL);
 
   /* Unicode in group name */
   data = "[\xc2\xbd]\n"
@@ -1007,25 +1025,19 @@ test_group_names (void)
   keyfile = g_key_file_new ();
   g_key_file_set_string (keyfile, "a[b", "key1", "123");
   value = g_key_file_get_string (keyfile, "a[b", "key1", &error);
-  check_error (&error, 
-               G_KEY_FILE_ERROR,
-               G_KEY_FILE_ERROR_GROUP_NOT_FOUND);  
+  check_log (G_LOG_LEVEL_CRITICAL);
   g_key_file_free (keyfile);  
 
   keyfile = g_key_file_new ();
   g_key_file_set_string (keyfile, "a]b", "key1", "123");
   value = g_key_file_get_string (keyfile, "a]b", "key1", &error);
-  check_error (&error, 
-               G_KEY_FILE_ERROR,
-               G_KEY_FILE_ERROR_GROUP_NOT_FOUND);  
+  check_log (G_LOG_LEVEL_CRITICAL);
   g_key_file_free (keyfile);  
 
   keyfile = g_key_file_new ();
   g_key_file_set_string (keyfile, "a\tb", "key1", "123");
   value = g_key_file_get_string (keyfile, "a\tb", "key1", &error);
-  check_error (&error, 
-               G_KEY_FILE_ERROR,
-               G_KEY_FILE_ERROR_GROUP_NOT_FOUND);  
+  check_log (G_LOG_LEVEL_CRITICAL);
   g_key_file_free (keyfile);  
 
   keyfile = g_key_file_new ();
@@ -1043,14 +1055,13 @@ test_key_names (void)
   gchar *value;
 
   /* [ in key name */
+  log_flags = 0;
   data = "[a]\n"
          "key[=123\n";
   keyfile = g_key_file_new ();
   g_key_file_load_from_data (keyfile, data, -1, 0, &error);
   g_key_file_free (keyfile);  
-  check_error (&error, 
-	       G_KEY_FILE_ERROR,
-	       G_KEY_FILE_ERROR_PARSE);
+  check_log (G_LOG_LEVEL_CRITICAL);
 
   /* control char in key name */
   data = "[a]\n"
@@ -1058,9 +1069,7 @@ test_key_names (void)
   keyfile = g_key_file_new ();
   g_key_file_load_from_data (keyfile, data, -1, 0, &error);
   g_key_file_free (keyfile);  
-  check_error (&error, 
-	       G_KEY_FILE_ERROR,
-	       G_KEY_FILE_ERROR_PARSE);
+  check_log (G_LOG_LEVEL_CRITICAL);
 
   /* Unicode in key name */
   data = "[a]\n"
@@ -1074,36 +1083,28 @@ test_key_names (void)
   g_key_file_set_string (keyfile, "a", "x", "123");
   g_key_file_set_string (keyfile, "a", "key=", "123");
   value = g_key_file_get_string (keyfile, "a", "key=", &error);
-  check_error (&error, 
-               G_KEY_FILE_ERROR,
-               G_KEY_FILE_ERROR_KEY_NOT_FOUND);  
+  check_log (G_LOG_LEVEL_CRITICAL);
   g_key_file_free (keyfile);  
 
   keyfile = g_key_file_new ();
   g_key_file_set_string (keyfile, "a", "x", "123");
   g_key_file_set_string (keyfile, "a", "key[", "123");
   value = g_key_file_get_string (keyfile, "a", "key[", &error);
-  check_error (&error, 
-               G_KEY_FILE_ERROR,
-               G_KEY_FILE_ERROR_KEY_NOT_FOUND);  
+  check_log (G_LOG_LEVEL_CRITICAL);
   g_key_file_free (keyfile);  
 
   keyfile = g_key_file_new ();
   g_key_file_set_string (keyfile, "a", "x", "123");
   g_key_file_set_string (keyfile, "a", "key\tfoo", "123");
   value = g_key_file_get_string (keyfile, "a", "key\tfoo", &error);
-  check_error (&error, 
-               G_KEY_FILE_ERROR,
-               G_KEY_FILE_ERROR_KEY_NOT_FOUND);  
+  check_log (G_LOG_LEVEL_CRITICAL);
   g_key_file_free (keyfile);  
 
   keyfile = g_key_file_new ();
   g_key_file_set_string (keyfile, "a", "x", "123");
   g_key_file_set_string (keyfile, "a", " key", "123");
   value = g_key_file_get_string (keyfile, "a", " key", &error);
-  check_error (&error, 
-               G_KEY_FILE_ERROR,
-               G_KEY_FILE_ERROR_KEY_NOT_FOUND);  
+  check_log (G_LOG_LEVEL_CRITICAL);
   g_key_file_free (keyfile);  
 
   keyfile = g_key_file_new ();
@@ -1177,14 +1178,6 @@ test_duplicate_groups2 (void)
   g_key_file_free (keyfile);  
 }
 
-static void 
-log_func (const gchar   *log_domain,
-          GLogLevelFlags log_level,
-          const gchar   *message,
-          gpointer       user_data)
-{
-}
-
 int
 main (int argc, char *argv[])
 {
